include 'Vram.inc'
include 'ZeroPage.inc'

ext InitVram_, UpdateSprites
ext InitSound_, SoundHandler

AsciiArea equ PatternRam+8*32
IntervalN equ $4295*6/5
IntervalP equ $4025*6/5

dseg
RegSave:
    defs $20
IrqSave:
    defw 0
NmiSave:
    defw 0
TimerCount: 
    defb 0

cseg
Init_:   public Init_
    lda $dd00 ; Port A, serial bus access. Bits:
    and #$fc
    ora #$01 ; 1: Bank #2, $8000-$BFFF, 32768-49151
    sta $dd00

    lda $d018
    and #$f1
    ora #$02
    sta $d018

    lda #0
    sta $d020 ; Border color
    sta $d021 ; Background color

    lda #$ff
    sta $d015 ; Sprite enable register	

    jsr InitVram_
    
    lda #0 | sta TimerCount

    sei
        lda $0314
        sta IrqSave
        lda $0315
        sta IrqSave+1
        lda #low IrqHandler
        sta $0314
        lda #high IrqHandler
        sta $0315

        lda #255
        sta $D012 ; Raster line to generate interrupt
        lda #1 ; Raster interrupt enabled
        sta $D019
        sta $D01A ; Interrupt control register. Bits
        lda #$7f
        sta $DC0D
        
        lda $0318
        sta NmiSave
        lda $0319
        sta NmiSave+1
        lda #low NmiHandler
        sta $0318
        lda #high NmiHandler
        sta $0319

        lda $02A6
        and 1
        if eq 
            lda #low IntervalN
            sta $dd04
            lda #high IntervalN
            sta $dd05
        else
            lda #low IntervalP
            sta $dd04
            lda #high IntervalP
            sta $dd05
        endif

        lda #$81 ; Enable NMI generated by timer A underflow.
        sta $DD0D ; Interrupt control and status register
        lda $dd0e
	    ora #$11
	    sta $dd0e
        sta $02A1
    cli

    jsr InitSound_

    ; lda $D011
    ; ora #$10
    ; sta $D011
rts


IrqHandler: public IrqHandler
    lda $D019
    and #$01
    if ne
        sta $D019
        jsr UpdateSprites
    endif
jmp (IrqSave)


NmiHandler: public NmiHandler
    pha
    txa | pha
    tya | pha
        lda $DD0D
        and #1
        if ne            
            inc TimerCount
            jsr SoundHandler

            lda #$81 ; Enable NMI generated by timer A underflow.
            sta $DD0D ; Interrupt control and status register
            lda $dd0e
            ora #$11
            sta $dd0e
            sta $02A1
        endif
    pla | tay
    pla | tax
    pla
    cli
rti
jmp (NmiSave)



; void WaitTimer(byte t);
dseg
WaitTimer_@Param0: public WaitTimer_@Param0
    defb 0
cseg
WaitTimer_: public WaitTimer_
    pha
        do
            lda TimerCount
            cmp WaitTimer_@Param0
        while cc | wend
        sei
            lda #0 | sta TimerCount
        cli
    pla
rts